module ad7606_driver(
	Clk,
	Reset_n,
	Speed_Set,
	
	Dout_A,
	Dout_B,
	ad7606_cs_n_o,
	ad7606_rd_n_o,
	ad7606_os_o,
	ad7606_convst_o,
	ad7606_db15,
	ad7606_db14_9,
	ad7606_db6_0,
	ad7606_frstdata,
	ad7606_reset_o,
 
	data_flag,
	data_a,
	data_b,
	
	data_1,
	data_2, 
	data_3, 
	data_4, 
	data_5, 
	data_6, 
	data_7, 
	data_8
);

	input Clk;
	input Reset_n;
	input [24:0]Speed_Set;
	
	input Dout_A;//串行输入A，输入1——4通道数据
	input Dout_B;//串行输入B，输入5——8通道数据
	input ad7606_frstdata;//串行模式下在Dout_A输出V1（通道一）的值时为高电平
	output reg ad7606_cs_n_o;//在串行模式使能串行数据帧传输
	output wire [2:0] ad7606_os_o; 
	output reg ad7606_convst_o;//转换有效信号
	output reg ad7606_rd_n_o;//输出到模块作为模块时钟
	output ad7606_db15;
	output [4:0]ad7606_db14_9;
    output [6:0]ad7606_db6_0;
	output reg ad7606_reset_o;
	
	output reg [15:0]data_a,data_b;
	output reg [7:0]data_flag;
	
	output [15:0]data_1,data_2,data_3,data_4,data_5,data_6,data_7,data_8;
	
	reg [15:0]data1,data2;

	assign ad7606_db15 = 0;//模式选择，串行模式为0
	assign ad7606_db14_9 = 0;
	assign ad7606_db6_0 = 0;
	assign ad7606_os_o = 0;	//不使用过采样
	
	parameter DIV_PARAM = 2;
	
	reg SCLK2X;//2倍SCLK的采样时钟
	reg [7:0]DIV_CNT;//分频计数器
	reg En;
	
	//生成2倍SCLK使能时钟
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		SCLK2X  <= 1'b0;
	else if(DIV_CNT == (DIV_PARAM - 1'b1))
		SCLK2X  <= 1'b1;
	else
		SCLK2X  <= 1'b0;
		
	//生成2倍SCLK使能时钟计数器
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		DIV_CNT  <= 8'd0;
	else if(En)begin
		if(DIV_CNT == (DIV_PARAM - 1'b1))
			DIV_CNT  <= 8'd0;
		else 
			DIV_CNT  <= DIV_CNT + 1'b1;
	end else	
		DIV_CNT  <= 8'd0;
		
	reg [7:0]SCLK_CNT;
	reg [1:0]cnt_speed;
	
	//生成序列计数器
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		SCLK_CNT  <= 8'd0;
	else if(SCLK2X)begin
		if(SCLK_CNT == 8'd130)
			SCLK_CNT  <= 8'd0;
		else if(En)begin
			SCLK_CNT  <= SCLK_CNT + 1'd1;
		end
	end 
	else
		SCLK_CNT  <= SCLK_CNT;	
		
	reg [1:0]state;	
	reg [24:0]cnt;
	
	//采样间隔
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		cnt <= 0;
	else if(cnt == Speed_Set)
		cnt <= 0;
	else if(!En)begin
		cnt <= cnt + 1'b1;
	end
		
	wire trig = cnt == Speed_Set;
	
	//采样控制
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)begin
		state<=0;
		En <= 0;
	end
	else begin
		case(state)
			0:
				if(trig)begin
					state <= 1;
					En <= 1;
				end
			1:
				if(SCLK_CNT == 8'd130)begin
					state <= 0;
					En <= 0;
				end
			default:;
		endcase
	end
	
	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)
		data_flag <= 0;
	else begin
		data_flag[0] <= SCLK_CNT == 130;
		data_flag[1] <= SCLK_CNT == 35 ;
		data_flag[2] <= SCLK_CNT == 67 ;
		data_flag[3] <= SCLK_CNT == 99 ;
		data_flag[4] <= SCLK_CNT == 130;
		data_flag[5] <= SCLK_CNT == 35;
		data_flag[6] <= SCLK_CNT == 67;
		data_flag[7] <= SCLK_CNT == 99;	
	end
	
	assign data_1 = (SCLK_CNT == 130)? data_a:data_1;
	assign data_2 = (SCLK_CNT == 35)? data_a:data_2;
	assign data_3 = (SCLK_CNT == 67)? data_a:data_3;
	assign data_4 = (SCLK_CNT == 99)? data_a:data_4;
	assign data_5 = (SCLK_CNT == 130)? data_b:data_5;
	assign data_6 = (SCLK_CNT == 35)? data_b:data_6;
	assign data_7 = (SCLK_CNT == 67)? data_b:data_7;
	assign data_8 = (SCLK_CNT == 99)? data_b:data_8;

	always@(posedge Clk or negedge Reset_n)
	if(!Reset_n)begin
		ad7606_cs_n_o <= 1;
		data_a <= 16'd0;
		data_b <= 16'd0;
		ad7606_convst_o <= 1;
		ad7606_reset_o <= 0;
		
		data1 <= 0;
		data2 <= 0;
	end

	else begin
		case(SCLK_CNT)
			0:
				begin
					ad7606_cs_n_o <= 1;
					data_a <= data_a;
					data_b <= data_b;
					ad7606_convst_o <= 0;
					ad7606_rd_n_o <= 1;
				end

			1:begin 
				   ad7606_convst_o <= 1;
					ad7606_cs_n_o <= 0;
					ad7606_rd_n_o <= 1;
			  end
			2:begin 
					ad7606_rd_n_o <= 1;
					ad7606_cs_n_o <= 0;
			  end
			  
			4 ,36,68,100,
			6 ,38,70,102,
			8 ,40,72,104,
			10,42,74,106,
			12,44,76,108,
			14,46,78,110,
			16,48,80,112,
			18,50,82,114,
			20,52,84,116,
			22,54,86,118,
			24,56,88,120,
			26,58,90,122,
			28,60,92,124,
			30,62,94,126,
			32,64,96,128:	
			begin        
				ad7606_rd_n_o <= 1;
			end
			
			3, 35,67,99:begin data1[15]<= Dout_A;data2[15]<= Dout_B;ad7606_rd_n_o <= 0;end
			5, 37,69,101:begin data1[14]<= Dout_A;data2[14]<= Dout_B;ad7606_rd_n_o <= 0;end
			7, 39,71,103:begin data1[13]<= Dout_A;data2[13]<= Dout_B;ad7606_rd_n_o <= 0;end	
			9, 41,73,105:begin data1[12]<= Dout_A;data2[12]<= Dout_B;ad7606_rd_n_o <= 0;end		
			11,43,75,107:begin data1[11]<= Dout_A;data2[11]<= Dout_B;ad7606_rd_n_o <= 0;end			
			13,45,77,109:begin data1[10]<= Dout_A;data2[10]<= Dout_B;ad7606_rd_n_o <= 0;end			
			15,47,79,111:begin data1[9]<= Dout_A;data2[9]<= Dout_B;ad7606_rd_n_o <= 0;end			
			17,49,81,113:begin data1[8]<= Dout_A;data2[8]<= Dout_B;ad7606_rd_n_o <= 0;end			
			19,51,83,115:begin data1[7]<= Dout_A;data2[7]<= Dout_B;ad7606_rd_n_o <= 0;end		
			21,53,85,117:begin data1[6]<= Dout_A;data2[6]<= Dout_B;ad7606_rd_n_o <= 0;end		
			23,55,87,119:begin data1[5]<= Dout_A;data2[5]<= Dout_B;ad7606_rd_n_o <= 0;end		
			25,57,89,121:begin data1[4]<= Dout_A;data2[4]<= Dout_B;ad7606_rd_n_o <= 0;end		
			27,59,91,123:begin data1[3]<= Dout_A;data2[3]<= Dout_B;ad7606_rd_n_o <= 0;end	
			29,61,93,125:begin data1[2]<= Dout_A;data2[2]<= Dout_B;ad7606_rd_n_o <= 0;end
			31,63,95,127:begin data1[1]<= Dout_A;data2[1]<= Dout_B;ad7606_rd_n_o <= 0;end
			33,65,97,129:begin data1[0]<= Dout_A;data2[0]<= Dout_B;ad7606_rd_n_o <= 0;end
			
			34,66,98,130:begin data_a <= data1;data_b <= data2;ad7606_rd_n_o <= 1;end
	
			default:
				begin
					ad7606_cs_n_o <= 1;
					data_a <= data_a;
					data_b <= data_b;
					ad7606_convst_o <= 1;
					ad7606_reset_o <= 0;
					
					data1 <= data1;
					data2 <= data2;
				end
		endcase
	end					
endmodule
	
