 module ad7606_uart(
	clk25m,
	Reset_n,
	Rs232_Tx,
	Rs232_Rx,
	led,
	
	ad7606_reset_o,
	ad7606_cs_n_o,   
	ad7606_rd_n_o,       
	ad7606_os_o,       
	ad7606_convst_o,
	ad7606_db15,
	ad7606_db14_9,
	ad7606_db6_0,
	ad7606_frstdata,
	
	Dout_A,
	Dout_B
);
    
    input  wire 		 clk25m;
	input  wire 		 Reset_n;

    input  wire 		 Rs232_Rx;
	output wire          Rs232_Tx;
	output wire 		 led;
        
	input  wire 		 ad7606_frstdata;
	input 				 Dout_A;
	input 				 Dout_B;
  	
	output wire [2:0]  ad7606_os_o;                   
	output wire        ad7606_convst_o;
	output wire        ad7606_db15;
    output wire        ad7606_cs_n_o;         
	output wire        ad7606_rd_n_o;
	output wire [14:9] ad7606_db14_9;
	output wire [6:0]  ad7606_db6_0;
	output wire        ad7606_reset_o;

	wire fifowrreq;
	wire [15:0]fifowrdata;
	wire [7:0]uart_tx_data;
	wire uart_send_en;
	wire uart_tx_done;       
	wire key_flag;
	wire key_state; 
	wire fifordempty;
	wire fifordreq;
	wire [15:0]fiforddata;
	
	wire [7:0]rx_data;
	wire rx_done;
	
	wire [7:0]ChannelSel;
	wire [15:0]DataNum;
	wire [31:0]ADC_Speed_Set;
	
	wire [7:0]cmd_addr;
	wire [31:0]cmd_data;
	wire cmdvalid;    
    
	wire RestartReq;
	wire [7:0]adc_data_flag;
	wire [15:0]adc_data_mult_ch;

	wire clk50M;
	wire PLL_Locked;
	PLL_FREQ_30K PLL(
		.CLKI(clk25m), 
		.CLKOP(), 
		.CLKOS(clk50M),
		.LOCK(PLL_Locked)
	);
	wire system_rst_n;
	assign system_rst_n = Reset_n && PLL_Locked;	

	
	assign led = system_rst_n;

	uart_byte_rx uart_byte_rx(
		.Clk(clk50M),
		.Reset_n(system_rst_n),
		.Baud_Set(4),
		.uart_rx(Rs232_Rx),
		.Data(rx_data),
		.Rx_Done(rx_done)  
    );
	
	uart_cmd uart_cmd(
        .Clk(clk50M),
        .Reset_n(system_rst_n),
        .rx_data(rx_data),
        .rx_done(rx_done),
        .address(cmd_addr),
        .data(cmd_data),
        .cmdvalid(cmdvalid)
    );
    
    cmd_ctrl cmd_ctrl(
       .Clk(clk50M),
       .Reset_n(system_rst_n),
       .cmd_data(cmd_data),
       .cmd_addr(cmd_addr),
       .cmdvalid(cmdvalid),
       .RestartReq(RestartReq),
       .ChannelSel(ChannelSel),
       .DataNum(DataNum),
       .ADC_Speed_Set(ADC_Speed_Set)
       );
	 
	 wire [15:0]data_a;
	 wire [15:0]data_b;
	 
	 ad7606_driver ad7606_driver(
        .Clk(clk50M),
        .Reset_n(system_rst_n),
        .Speed_Set(ADC_Speed_Set),  
        .ad7606_cs_n_o(ad7606_cs_n_o), 
        .ad7606_rd_n_o(ad7606_rd_n_o), 
        .ad7606_os_o(ad7606_os_o), 
        .ad7606_convst_o(ad7606_convst_o), 
		.ad7606_db15(ad7606_db15),
		.ad7606_db6_0(ad7606_db6_0),
		.ad7606_db14_9(ad7606_db14_9),
		.ad7606_frstdata(ad7606_frstdata),
		.ad7606_reset_o(ad7606_reset_o), 
		
		.Dout_A(Dout_A),
		.Dout_B(Dout_B),
		
		.data_flag(adc_data_flag),
		.data_a(data_a),
		.data_b(data_b),
		.data_1(),
		.data_2(), 
		.data_3(), 
		.data_4(), 
		.data_5(), 
		.data_6(), 
		.data_7(), 
		.data_8()
    );
	 	 

	FIFO FIFO( 
		.Data(fifowrdata), 
		.WrClock(clk50M), 
		.RdClock(clk50M), 
		.WrEn(fifowrreq), 
		.RdEn(fifordreq),  
		.Reset(!system_rst_n), 
		.RPReset(), 
		.Q(fiforddata), 
		.Empty(fifordempty), 
		.Full(), 
		.AlmostEmpty(), 
		.AlmostFull()
	);

	 reg [7:0]adc_data_flag_0,adc_data_flag_1;
	 reg [15:0]data_a0,data_a1,data_b0,data_b1;
	 always@(posedge clk50M)
	 begin
		adc_data_flag_0 <= adc_data_flag;
		adc_data_flag_1 <= adc_data_flag_0;
		
		data_a0 <= data_a;
		data_a1 <= data_a0;
		data_b0 <= data_b;
		data_b1 <= data_b0;	
	 end
	   
    adc_write_ctrl adc_write_ctrl(
        .Clk(clk50M),
        .Reset_n(system_rst_n),
        .DataNum(DataNum),
        .RestartReq(RestartReq),
        .ChannelSel(ChannelSel),
        .fifowrreq(fifowrreq),
        .fifowrdata(fifowrdata),
        .fifowrfull(),
        .adc_data_flag(adc_data_flag_1),
		.data_a(data_a1),
		.data_b(data_b1)
    );

    uart_byte_tx uart_byte_tx(
        .Clk(clk50M),       			//50M时钟输入
        .Rst_n(system_rst_n),     		//模块复位
        .data_byte(uart_tx_data),//待传输8bit数据
        .send_en(uart_send_en),	//发送使能
        .baud_set(4),  				//波特率设置
        .Rs232_Tx(Rs232_Tx),  	//Rs232输出信号
        .Tx_Done(uart_tx_done),	//一次发送数据完成标志
        .uart_state() 				//发送数据状态
    );
    
    uart_send_ctrl uart_send_ctrl(
        .Clk(clk50M),
        .Reset_n(system_rst_n),
        .uart_tx_data(uart_tx_data),
        .uart_send_en(uart_send_en),
        .uart_tx_done(uart_tx_done),
        .fifordempty(fifordempty),
        .fifordreq(fifordreq),
        .fiforddata(fiforddata)
    );
    

endmodule
